#include "main.h"
#include <util/delay.h>

void System_Init(void)
{
	// SysTimer init 1ms
	TCNT0 = 193;
	TCCR0B |= (1 << CS01) | (1 << CS00);
	TIMSK0 |= (1 << TOIE0);

	SPI_Init();
	UART_Init(BAUDRATE);

	PCICR  |= (1 << PCIE2);
	PCMSK2 |= (1 << PCINT18);

	sei();
}

void SysTick_Handler(void)
{

}

ISR(TIMER0_OVF_vect)
{
	TCNT0 = 131;
	SysTick_Handler();
}

void UART_PrintRegister(unsigned char* src, unsigned char addr)
{
	UART_SendString("-------- CS8422 Registers -------\r\n");

	UART_printf("Addr 0x%02X : Value 0x%02X 0b", addr, src[addr - 1]);

	for (unsigned char j = 0; j < 8; j++)
	{
		(((src[addr - 1] << j) & 0x80) == 0x80) ? UART_SendString("1") : UART_SendString("0");
	}

	UART_SendString("\r\n");
	UART_SendString("---------------------------------\r\n");
}

void UART_PrintNRegisters(unsigned char* src, unsigned char start_addr, unsigned char count)
{
	UART_SendString("-------- CS8422 Registers -------\r\n");

	for (unsigned char i = start_addr; i < start_addr + count; i++)
	{
		UART_printf("Addr 0x%02X : Value 0x%02X 0b", i, src[i - 1]);

		for (unsigned char j = 0; j < 8; j++)
		{
			(((src[i - 1] << j) & 0x80) == 0x80) ? UART_SendString("1") : UART_SendString("0");
		}

		UART_SendString("\r\n");
	}
	UART_SendString("---------------------------------\r\n");
}

unsigned char TX_buf[2] = {0};

unsigned char data = 0;

int main(void)
{
	System_Init();

	data = 0xA4;
	CS8422_WriteBlock(&data, REG_03_RESEIVER_INPUT_CTRL, 1); // RX single ended, select RX1, mode 2

	data = 0x46;
	CS8422_WriteBlock(&data, REG_08_SRC_SAO_CLK_CTRL, 1); // OLRCK = MCLK/256, OMCLK = XTI-XTO, SRC_MCLK = XTI-XTO

	data = 0x48;
	CS8422_WriteBlock(&data, REG_09_RMCK_RATIO_CTRL_MISC, 1); // RMCK = 256Fs, Soft mute enabled

	data = 0x12;
	CS8422_WriteBlock(&data, REG_0A_DATA_ROUTING_CTRL, 1); // OUT1 - SRC, OUT2 - SPDIF, SRC input - SPDIF

	data = 0x84;
	CS8422_WriteBlock(&data, REG_0C_SAO_1_DATA_FMT, 1); // OUT1 - I2S 24 bit master

	data = 0x84;
	CS8422_WriteBlock(&data, REG_0D_SAO_2_DATA_FMT, 1); // OUT2 - I2S 24 bit master

	data = 0x00;
	CS8422_WriteBlock(&data, REG_02_CLK_CTRL, 1); // PDN OFF

    while (1)
    {
		CS8422_ReadBlock(REG_01_CHIP_ID_VER, ALL_REGISTERS);
		UART_PrintNRegisters(CS8422_buf, REG_01_CHIP_ID_VER, 44);
		_delay_ms(2000);
    }
}

